#ifdef __linux__
#pragma once
#include "MemPool.h"
#include "AutoLock.h"
#include <stdio.h>
#include <pthread.h>
#include <unistd.h>


unsigned char g_bIsMemPoolWorked = 0;
static void *RebindThread(void *arg);
/***************************************************
linux版本的锁
****************************************************/
class CLinuxLock : public IBaseLock
{
public:
	CLinuxLock(void)
	{
		pthread_mutex_init(&mutex,NULL);
	}

	~CLinuxLock(void)
	{
		pthread_mutex_destroy(&mutex);
	}

	void Lock()
	{
		pthread_mutex_lock(&mutex);
	}

	void UnLock()
	{
		pthread_mutex_unlock(&mutex);
	}

private:
	pthread_mutex_t mutex;
};

class CLinuxMemPool : public CMemPool
{
public:
	static CLinuxMemPool* GetSingletonMemPool()
	{
		static CLinuxMemPool mem;
		return &mem;
	}
private:
	CLinuxMemPool(void)
	{
		g_bIsMemPoolWorked = 1;
		//g_hReBindThread = CreateEvent(NULL,TRUE,FALSE,NULL);
		for (int i = 0;i < MEMPOOL_MAX_COUNT;i++)
		{
			m_pMemPoolLock[i] = new CLinuxLock;
		}

		pthread_t id;
		int ret = pthread_create(&id, NULL, RebindThread, NULL);
		//g_hThreadHandle = (HANDLE)_beginthreadex(NULL,NULL,RebindMemPoolThread,this,NULL,NULL);
	}
	
	~CLinuxMemPool(void)
	{
		g_bIsMemPoolWorked = 0;

		FreeIdleTrunk();

		for (int i = 0;i < MEMPOOL_MAX_COUNT;i++)
		{
			delete m_pMemPoolLock[i];
		}
	}
};

CLinuxMemPool *g_pMemPool = CLinuxMemPool::GetSingletonMemPool();
/*******************************************************
重新整理内存池，检查空闲内存的闲置时间，超过MEMPOOL_TIMELIM毫秒就释放
同时也整理动态数组m_MemPoolTable避免占用太多无用内存
********************************************************/
static void *RebindThread(void *arg)
{
	static unsigned char s_bIsBusy = 0;
	if (1 == s_bIsBusy)
		return 0;

	s_bIsBusy = 1;

	while (1)
	{
		usleep(5000000);
		if (1 == g_bIsMemPoolWorked)
		{
			g_pMemPool->RebindMemPool();
		}
		else
		{
			break;
		}
	}
	s_bIsBusy = 0;
	return 0 ;
}


#endif
