/***************************************************************************************
* AUTHOR : 心爱永远
* DATE   : 2014-01-29
* MODULE : MemPool
* EMAIL  : 274536233@qq.com
* BLOG   : http://www.cnblogs.com/mod109
* Copyright (C) 2014 心爱永远.
****************************************************************************************
内存池测试版V0.2修改日志(修改时间2014-01-30)：
1、修正在内存池列为NULL时有可能崩溃的bug
2、提供用户检测内存泄漏的功能
3、增加自动释放长时间空闲内存功能
4、修改部分类型转换过程，关闭因reinterpret_cast产生的警告

内存池测试版V0.3修改日志(修改时间2014-01-31)：
1、修改部分变量名
2、优化部分判断逻辑
3、修改申请内存的逻辑结构，进一步优化锁竞争

内存池测试版V0.4修改日志(修改时间2014-01-31)：
1、增加自动收缩内存功能，避免闲置太多浪费内存
2、修正自动释放长时间不用内存时到边界崩溃的bug

内存池V1.0修改日志(修改时间2014-02-03)：
1、修正增加MEMPOOL_MAX数值有可能崩溃的bug
2、修改动态数组起始长度以便于节约内存

内存池V1.1修改日志(修改时间2014-02-07)：
1、优化回收内存时的部分策略
2、修改程序退出时内存池析构的部分策略

内存池V2.0修改日志(修改时间2015-11-28)：
1、修改了锁策略
2、修改代码逻辑实现跨平台编译
****************************************************************************************/
#pragma once
#include "DynamicAry.h"
#include "AutoLock.h"

#ifdef _WIN32
#pragma warning(disable : 4311 4312)
#endif

/****************************************************************************************/
#define  ch_TRUE   '1'
#define  ch_FLASE  '0'
#define  ch_SIZE   sizeof(MemPoolData)
//最大申请内存，对齐后如果照过这个数值则按照需要大小调用mallloc申请
#define MEMPOOL_MAX               512
//内存申请时对齐，默认按照8的倍数
#define MEMPOOL_ALIGN             8
//内存池伸缩数组元素个数
#define MEMPOOL_MAX_COUNT         MEMPOOL_MAX/MEMPOOL_ALIGN
//内存池对齐计算宏
#define APR_ALIGN(size, boundary) (((size) + ((boundary) - 1)) & ~((boundary) - 1))
//内存池闲置时间控制，默认为闲置五分钟则释放
#define MEMPOOL_TIMELIM           300
/*************************************************************
内存池管理数据结构，分别标记该块内存使用状态，
在内存池中的外部索引，内部索引，闲置起始时间。
内存池总体数据结构为m_MemPoolTable[MEMPOOL_MAX_COUNT]
chOutIndex标记则是记录该块内存属于第几个m_MemPoolTable[x]
chInIndexz标记则是该内存属于m_MemPoolTable[x]中的第几个位置
**************************************************************/
struct MemPoolData
{
	unsigned int    dwMemTrunkTicket;
	unsigned char   chbIsMemTrunkUsed;
	unsigned char   chOutIndex;
	unsigned char   chInIndex;
	
	int GetOutIndex()
	{
		return chOutIndex-'0';
	}
	void SetOutIndex(int iIndex)
	{ 
		char chIndex = iIndex + '0';
		chOutIndex=chIndex;
	}
	int GetInIndex()
	{
		return chInIndex-'0';
	}
	void  SetInIndex(int iIndex)
	{
		char chIndex = iIndex + '0';
		chInIndex=chIndex;
	}
	MemPoolData operator=(MemPoolData data)
	{
		dwMemTrunkTicket  = data.dwMemTrunkTicket;
		chInIndex = data.chInIndex;
		return *this;
	}
};
/*************************************************************
内存池主体结构，由以下两个变量组成
IBaseLock *m_pMemPoolLock[MEMPOOL_MAX_COUNT];
CDynamicAry<LPVOID> m_MemPoolTable[MEMPOOL_MAX_COUNT];
每一个锁m_csMemPoollock[i]锁住一个动态数组中的一项m_MemPoolTable[i]
每一个m_MemPoolTable[i]叫做一个内存池列。

申请内存时始终从m_MemPoolTable[i]的第一个元素中获取
如内存已被使用则调用malloc申请
如未使用则将其内存快返回，并将其与闲置的最后一个元素交换保证保证前一部分元素未使用后一部分被使用
**************************************************************/
class CMemPool
{
public:
	CMemPool();
	~CMemPool();
	//申请和释放内存
	void* MallocBuffer(size_t nSize);
	void  FreeBuffer(void* pBuffer);
public:
	void RebindMemPool();

	void FreeIdleTrunk();

	int SwapPoolData(int dwOutIndex,int dwIndex1,int dwIndex2);

	IBaseLock *m_pMemPoolLock[MEMPOOL_MAX_COUNT];

	CDynamicAry<void*> m_MemPoolTable[MEMPOOL_MAX_COUNT];
};
