#ifdef _WIN32
#pragma once
#include "mempool.h"
#include "AutoLock.h"
#include <Windows.h>
#include <process.h>

HANDLE g_hReBindThread;
HANDLE g_hThreadHandle;
unsigned char g_bIsMemPoolWorked = 0;
UINT WINAPI RebindMemPoolThread(LPVOID lpParam);
/************************************************
windowƽ版的锁策略
*************************************************/
class CWindowLock : public IBaseLock
{
public:
	CWindowLock(void)
	{
		InitializeCriticalSection(&m_csMemPoollock);  
	}

	~CWindowLock(void)
	{
		DeleteCriticalSection(&m_csMemPoollock); 
	}

	void Lock()
	{
		EnterCriticalSection(&m_csMemPoollock);
	}

	void UnLock()
	{
		LeaveCriticalSection(&m_csMemPoollock);
	}

private:
	CRITICAL_SECTION    m_csMemPoollock;
};

class CWindowMemPool : public CMemPool
{
public:
	static CWindowMemPool* GetSingletonMemPool()
	{
		static CWindowMemPool mem;
		return &mem;
	}
private:
	CWindowMemPool(void)
	{
		g_bIsMemPoolWorked = 1;
		g_hReBindThread = CreateEvent(NULL,TRUE,FALSE,NULL);
		for (int i = 0;i < MEMPOOL_MAX_COUNT;i++)
		{
			m_pMemPoolLock[i] = new CWindowLock;
		}
		g_hThreadHandle = (HANDLE)_beginthreadex(NULL,NULL,RebindMemPoolThread,this,NULL,NULL);
	}
	
	~CWindowMemPool(void)
	{
		g_bIsMemPoolWorked = 0;
		SetEvent(g_hReBindThread);
		WaitForSingleObject(g_hThreadHandle,INFINITE);
		CloseHandle(g_hReBindThread);
		CloseHandle(g_hThreadHandle);

		FreeIdleTrunk();

		for (int i = 0;i < MEMPOOL_MAX_COUNT;i++)
		{
			delete m_pMemPoolLock[i];
		}
	}
};

/*******************************************************
重新整理内存池，检查空闲内存的闲置时间，超过MEMPOOL_TIMELIM毫秒就释放
同时也整理动态数组m_MemPoolTable避免占用太多无用内存
********************************************************/
UINT WINAPI RebindMemPoolThread(LPVOID lpParam)
{
	static unsigned char s_bIsBusy = 0;
	if (1 == s_bIsBusy)
		return 0;

	s_bIsBusy = 1;
	CWindowMemPool *pThis = (CWindowMemPool*)lpParam;

	while (1)
	{
		DWORD dwRet = WaitForSingleObject(g_hReBindThread,5000);
		if (1 == g_bIsMemPoolWorked)
		{
			pThis->RebindMemPool();
		}
		else
		{
			break;
		}
	}
	s_bIsBusy = 0;
	return 0;
}

CWindowMemPool *g_pMemPool = CWindowMemPool::GetSingletonMemPool();
#endif
